@@ -24,8 +24,7 @@
 #include "dh-settings.h"
 
 typedef struct {
-        GtkWidget     *view;
-        DhSettings    *settings;
+        GtkWidget *view;
 } DhAssistantPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (DhAssistant, dh_assistant, GTK_TYPE_APPLICATION_WINDOW);
@@ -60,34 +59,22 @@ static gboolean
 dh_assistant_configure_event (GtkWidget         *widget,
                               GdkEventConfigure *event)
 {
-        DhAssistant *assistant = DH_ASSISTANT (widget);
-        DhAssistantPrivate *priv = dh_assistant_get_instance_private (assistant);
+        DhSettings *settings;
+
+        settings = dh_settings_get_singleton ();
 
         dh_util_window_settings_save (GTK_WINDOW (widget),
-                                      dh_settings_peek_assistant_settings (priv->settings),
+                                      dh_settings_peek_assistant_settings (settings),
                                       FALSE);
+
         return GTK_WIDGET_CLASS (dh_assistant_parent_class)->configure_event (widget, event);
 }
 
-static void
-dh_assistant_dispose (GObject *object)
-{
-        DhAssistant *assistant = DH_ASSISTANT (object);
-        DhAssistantPrivate *priv = dh_assistant_get_instance_private (assistant);
-
-        g_clear_object (&priv->settings);
-
-        G_OBJECT_CLASS (dh_assistant_parent_class)->dispose (object);
-}
-
 static void
 dh_assistant_class_init (DhAssistantClass *klass)
 {
-        GObjectClass *object_class = G_OBJECT_CLASS (klass);
         GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-        object_class->dispose = dh_assistant_dispose;
-
         widget_class->key_press_event = dh_assistant_key_press_event;
         widget_class->configure_event = dh_assistant_configure_event;
 
@@ -101,52 +88,29 @@ static void
 dh_assistant_init (DhAssistant *assistant)
 {
         DhAssistantPrivate *priv = dh_assistant_get_instance_private (assistant);
+        DhSettings *settings;
 
         gtk_widget_init_template (GTK_WIDGET (assistant));
 
-        priv->settings = dh_settings_get ();
-
         g_signal_connect (priv->view, "open-uri",
                           G_CALLBACK (assistant_view_open_uri_cb),
                           assistant);
 
+        settings = dh_settings_get_singleton ();
+
         dh_util_window_settings_restore (GTK_WINDOW (assistant),
-                                         dh_settings_peek_assistant_settings (priv->settings),
+                                         dh_settings_peek_assistant_settings (settings),
                                          FALSE);
 }
 
-/**
- * dh_assistant_new:
- * @application: the parent #DhApp
- *
- * Create a new #DhAssistant object.
- *
- * Returns: a new #DhAssistant object
- */
 GtkWidget *
 dh_assistant_new (DhApp *application)
 {
-        GtkWidget          *assistant;
-        DhAssistantPrivate *priv;
-
-        assistant = g_object_new (DH_TYPE_ASSISTANT, "application", application, NULL);
-
-        priv = dh_assistant_get_instance_private (DH_ASSISTANT (assistant));
-        dh_assistant_view_set_book_manager (DH_ASSISTANT_VIEW (priv->view),
-                                            dh_app_peek_book_manager (application));
-
-        return assistant;
+        return g_object_new (DH_TYPE_ASSISTANT,
+                             "application", application,
+                             NULL);
 }
 
-/**
- * dh_assistant_search:
- * @assistant: a #DhAssistant object
- * @str: the search query
- *
- * Search for @str in the current assistant.
- *
- * Returns: %TRUE if @str was found, %FALSE otherwise
- */
 gboolean
 dh_assistant_search (DhAssistant *assistant,
                      const gchar *str)
