@@ -409,15 +409,15 @@ book_manager_is_book_disabled_in_conf (DhBookManager *book_manager,
                                        DhBook        *book)
 {
         DhBookManagerPrivate *priv = dh_book_manager_get_instance_private (book_manager);
-        const gchar *name;
+        const gchar *book_id;
         GSList *l;
 
-        name = dh_book_get_name (book);
+        book_id = dh_book_get_id (book);
 
         for (l = priv->books_disabled; l != NULL; l = l->next) {
-                gchar *cur_name = l->data;
+                gchar *cur_book_id = l->data;
 
-                if (g_strcmp0 (name, cur_name) == 0)
+                if (g_strcmp0 (book_id, cur_book_id) == 0)
                         return TRUE;
         }
 
@@ -456,11 +456,13 @@ book_manager_book_updated_cb (DhBook   *book,
                               gpointer  user_data)
 {
         DhBookManager *book_manager = user_data;
-        gchar         *book_path;
+        GFile *index_file;
+        gchar *book_path;
 
         /* When we update a book, we need to delete it and then
          * create it again. */
-        book_path = g_strdup (dh_book_get_path (book));
+        index_file = dh_book_get_index_file (book);
+        book_path = g_file_get_path (index_file);
         book_manager_book_deleted_cb (book, book_manager);
         book_manager_add_from_filepath (book_manager, book_path);
         g_free (book_path);
@@ -473,7 +475,7 @@ book_manager_find_book_in_disabled_list (GSList *books_disabled,
         GSList *li;
 
         for (li = books_disabled; li; li = g_slist_next (li)) {
-                if (g_strcmp0 (dh_book_get_name (book),
+                if (g_strcmp0 (dh_book_get_id (book),
                                (const gchar *)li->data) == 0) {
                         return li;
                 }
@@ -521,7 +523,7 @@ book_manager_book_disabled_cb (DhBook   *book,
          * disabled books list! */
         g_assert (li == NULL);
         priv->books_disabled = g_slist_append (priv->books_disabled,
-                                               g_strdup (dh_book_get_name (book)));
+                                               g_strdup (dh_book_get_id (book)));
         book_manager_store_books_disabled (book_manager);
 
         /* Decrement language count */
@@ -539,16 +541,19 @@ book_manager_add_from_filepath (DhBookManager *book_manager,
                                 const gchar   *book_path)
 {
         DhBookManagerPrivate *priv;
-        DhBook            *book;
-        gboolean           book_enabled;
+        GFile *index_file;
+        DhBook *book;
+        gboolean book_enabled;
 
         g_return_if_fail (book_manager);
         g_return_if_fail (book_path);
 
         priv = dh_book_manager_get_instance_private (book_manager);
 
-        /* Allocate new book struct */
-        book = dh_book_new (book_path);
+        index_file = g_file_new_for_path (book_path);
+        book = dh_book_new (index_file);
+        g_object_unref (index_file);
+
         if (book == NULL)
                 return;
 
@@ -560,11 +565,12 @@ book_manager_add_from_filepath (DhBookManager *book_manager,
                 return;
         }
 
-        /* Check if book with same bookname was already loaded in the manager
-         * (we need to force unique book names) */
+        /* Check if book with same ID was already loaded in the manager (we need
+         * to force unique book IDs).
+         */
         if (g_list_find_custom (priv->books,
                                 book,
-                                (GCompareFunc)dh_book_cmp_by_name)) {
+                                (GCompareFunc)dh_book_cmp_by_id)) {
                 g_object_unref (book);
                 return;
         }
