@@ -15,15 +15,13 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "dh-book.h"
-
 #include <glib/gi18n-lib.h>
-
 #include "dh-link.h"
 #include "dh-parser.h"
 #include "dh-util.h"
@@ -39,16 +37,16 @@
  * or `*.devhelp2`).
  */
 
-/* Timeout to wait for new events in the book so that
- * they are merged and we don't spam unneeded signals */
-#define EVENT_MERGE_TIMEOUT_SECS 2
+/* Timeout to wait for new events on the index file so that they are merged and
+ * we don't spam unneeded signals.
+ */
+#define EVENT_MERGE_TIMEOUT_SECS (2)
 
-/* Signals managed by the DhBook */
 enum {
-        BOOK_ENABLED,
-        BOOK_DISABLED,
-        BOOK_UPDATED,
-        BOOK_DELETED,
+        SIGNAL_BOOK_ENABLED,
+        SIGNAL_BOOK_DISABLED,
+        SIGNAL_BOOK_UPDATED,
+        SIGNAL_BOOK_DELETED,
         N_SIGNALS
 };
 
@@ -56,34 +54,29 @@ typedef enum {
         BOOK_MONITOR_EVENT_NONE,
         BOOK_MONITOR_EVENT_UPDATED,
         BOOK_MONITOR_EVENT_DELETED
-} DhBookMonitorEvent;
+} BookMonitorEvent;
 
-/* Structure defining basic contents to store about every book */
 typedef struct {
-        gchar        *index_file_path;
-        gchar        *name;
-        gchar        *title;
-        gchar        *language;
+        GFile *index_file;
 
-        /* The book tree of DhLink* */
-        GNode        *tree;
+        gchar *id;
+        gchar *title;
+        gchar *language;
 
-        /* List of DhLink* */
-        GList        *keywords;
+        /* The book tree of DhLink*. */
+        GNode *tree;
 
-        /* Generated list of keyword completions (gchar*) in the book */
-        GList        *completions;
+        /* List of DhLink*. */
+        GList *links;
 
-        /* Monitor of this specific book */
-        GFileMonitor *monitor;
+        /* Generated list of completions (gchar*) in the book. */
+        GList *completions;
 
-        /* Last received event */
-        DhBookMonitorEvent monitor_event;
+        GFileMonitor *index_file_monitor;
+        BookMonitorEvent last_monitor_event;
+        guint monitor_event_timeout_id;
 
-        /* ID of the event source */
-        guint         monitor_event_timeout_id;
-
-        guint         enabled : 1;
+        guint enabled : 1;
 } DhBookPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (DhBook, dh_book, G_TYPE_OBJECT);
@@ -97,7 +90,7 @@ dh_book_dispose (GObject *object)
 
         priv = dh_book_get_instance_private (DH_BOOK (object));
 
-        g_clear_object (&priv->monitor);
+        g_clear_object (&priv->index_file_monitor);
 
         if (priv->monitor_event_timeout_id != 0) {
                 g_source_remove (priv->monitor_event_timeout_id);
@@ -114,13 +107,13 @@ dh_book_finalize (GObject *object)
 
         priv = dh_book_get_instance_private (DH_BOOK (object));
 
-        _dh_util_free_book_tree (priv->tree);
-        g_list_free_full (priv->keywords, (GDestroyNotify)dh_link_unref);
-        g_list_free_full (priv->completions, g_free);
-        g_free (priv->language);
+        g_clear_object (&priv->index_file);
+        g_free (priv->id);
         g_free (priv->title);
-        g_free (priv->name);
-        g_free (priv->index_file_path);
+        g_free (priv->language);
+        _dh_util_free_book_tree (priv->tree);
+        g_list_free_full (priv->links, (GDestroyNotify)dh_link_unref);
+        g_list_free_full (priv->completions, g_free);
 
         G_OBJECT_CLASS (dh_book_parent_class)->finalize (object);
 }
@@ -135,9 +128,9 @@ dh_book_class_init (DhBookClass *klass)
 
         /**
          * DhBook::enabled:
-         * @book: the book on which the signal is emitted.
+         * @book: the #DhBook emitting the signal.
          */
-        signals[BOOK_ENABLED] =
+        signals[SIGNAL_BOOK_ENABLED] =
                 g_signal_new ("enabled",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
@@ -148,9 +141,9 @@ dh_book_class_init (DhBookClass *klass)
 
         /**
          * DhBook::disabled:
-         * @book: the book on which the signal is emitted.
+         * @book: the #DhBook emitting the signal.
          */
-        signals[BOOK_DISABLED] =
+        signals[SIGNAL_BOOK_DISABLED] =
                 g_signal_new ("disabled",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
@@ -161,9 +154,9 @@ dh_book_class_init (DhBookClass *klass)
 
         /**
          * DhBook::updated:
-         * @book: the book on which the signal is emitted.
+         * @book: the #DhBook emitting the signal.
          */
-        signals[BOOK_UPDATED] =
+        signals[SIGNAL_BOOK_UPDATED] =
                 g_signal_new ("updated",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
@@ -174,9 +167,9 @@ dh_book_class_init (DhBookClass *klass)
 
         /**
          * DhBook::deleted:
-         * @book: the book on which the signal is emitted.
+         * @book: the #DhBook emitting the signal.
          */
-        signals[BOOK_DELETED] =
+        signals[SIGNAL_BOOK_DELETED] =
                 g_signal_new ("deleted",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
@@ -192,118 +185,126 @@ dh_book_init (DhBook *book)
         DhBookPrivate *priv = dh_book_get_instance_private (book);
 
         priv->enabled = TRUE;
-        priv->monitor_event = BOOK_MONITOR_EVENT_NONE;
+        priv->last_monitor_event = BOOK_MONITOR_EVENT_NONE;
 }
 
 static gboolean
-book_monitor_event_timeout_cb (gpointer data)
+monitor_event_timeout_cb (gpointer data)
 {
-        DhBook *book = data;
+        DhBook *book = DH_BOOK (data);
         DhBookPrivate *priv = dh_book_get_instance_private (book);
-        DhBookMonitorEvent monitor_event = priv->monitor_event;
+        BookMonitorEvent last_monitor_event = priv->last_monitor_event;
 
         /* Reset event */
-        priv->monitor_event = BOOK_MONITOR_EVENT_NONE;
+        priv->last_monitor_event = BOOK_MONITOR_EVENT_NONE;
         priv->monitor_event_timeout_id = 0;
 
-        /* We'll get either is_deleted OR is_updated,
-         * not possible to have both or none */
-        switch (monitor_event)
+        /* We'll get either is_deleted OR is_updated, not possible to have both
+         * or none.
+         */
+        switch (last_monitor_event)
         {
         case BOOK_MONITOR_EVENT_DELETED:
-                /* Emit the signal, but make sure we hold a reference
-                 * while doing it */
+                /* Emit the signal, but make sure we hold a reference while
+                 * doing it.
+                 */
                 g_object_ref (book);
-                g_signal_emit (book, signals[BOOK_DELETED], 0);
+                g_signal_emit (book, signals[SIGNAL_BOOK_DELETED], 0);
                 g_object_unref (book);
                 break;
+
         case BOOK_MONITOR_EVENT_UPDATED:
-                /* Emit the signal, but make sure we hold a reference
-                 * while doing it */
+                /* Emit the signal, but make sure we hold a reference while
+                 * doing it.
+                 */
                 g_object_ref (book);
-                g_signal_emit (book, signals[BOOK_UPDATED], 0);
+                g_signal_emit (book, signals[SIGNAL_BOOK_UPDATED], 0);
                 g_object_unref (book);
                 break;
+
         case BOOK_MONITOR_EVENT_NONE:
         default:
                 break;
         }
 
-        /* book can be destroyed here */
+        /* book can be destroyed here. */
 
         return G_SOURCE_REMOVE;
 }
 
 static void
-book_monitor_event_cb (GFileMonitor      *file_monitor,
+index_file_changed_cb (GFileMonitor      *file_monitor,
                        GFile             *file,
                        GFile             *other_file,
                        GFileMonitorEvent  event_type,
-                       gpointer           user_data)
+                       DhBook            *book)
 {
-        DhBook     *book = user_data;
         DhBookPrivate *priv = dh_book_get_instance_private (book);
-        gboolean    reset_timer = FALSE;
+        gboolean reset_timeout = FALSE;
 
         /* CREATED may happen if the file is deleted and then created right
-         * away, as we're merging events.  Treat in the same way as a
-         * CHANGES_DONE_HINT.
+         * away, as we're merging events.
          */
-        if (event_type == G_FILE_MONITOR_EVENT_CREATED ||
-            event_type == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT) {
-                priv->monitor_event = BOOK_MONITOR_EVENT_UPDATED;
-                reset_timer = TRUE;
+        if (event_type == G_FILE_MONITOR_EVENT_CHANGED ||
+            event_type == G_FILE_MONITOR_EVENT_CREATED) {
+                priv->last_monitor_event = BOOK_MONITOR_EVENT_UPDATED;
+                reset_timeout = TRUE;
         } else if (event_type == G_FILE_MONITOR_EVENT_DELETED) {
-                priv->monitor_event = BOOK_MONITOR_EVENT_DELETED;
-                reset_timer = TRUE;
+                priv->last_monitor_event = BOOK_MONITOR_EVENT_DELETED;
+                reset_timeout = TRUE;
         }
 
-        /* Reset timer if any of the flags changed */
-        if (reset_timer) {
-                if (priv->monitor_event_timeout_id != 0) {
+        if (reset_timeout) {
+                if (priv->monitor_event_timeout_id != 0)
                         g_source_remove (priv->monitor_event_timeout_id);
-                }
+
                 priv->monitor_event_timeout_id = g_timeout_add_seconds (EVENT_MERGE_TIMEOUT_SECS,
-                                                                        book_monitor_event_timeout_cb,
+                                                                        monitor_event_timeout_cb,
                                                                         book);
         }
 }
 
 /**
  * dh_book_new:
- * @index_file_path: the path to the index file.
+ * @index_file: the index file.
  *
- * Returns: a new #DhBook object.
+ * Returns: (nullable): a new #DhBook object, or %NULL if parsing the index file
+ * failed.
  */
 DhBook *
-dh_book_new (const gchar *index_file_path)
+dh_book_new (GFile *index_file)
 {
         DhBookPrivate *priv;
-        DhBook     *book;
-        GFile      *index_file;
-        gchar      *language = NULL;
-        GError     *error = NULL;
+        DhBook *book;
+        gchar *language = NULL;
+        GError *error = NULL;
 
-        g_return_val_if_fail (index_file_path, NULL);
+        g_return_val_if_fail (G_IS_FILE (index_file), NULL);
 
         book = g_object_new (DH_TYPE_BOOK, NULL);
         priv = dh_book_get_instance_private (book);
 
-        index_file = g_file_new_for_path (index_file_path);
+        priv->index_file = g_object_ref (index_file);
 
-        /* Parse file storing contents in the book struct */
-        if (!dh_parser_read_file (index_file,
+        /* Parse file storing contents in the book struct. */
+        if (!dh_parser_read_file (priv->index_file,
                                   &priv->title,
-                                  &priv->name,
+                                  &priv->id,
                                   &language,
                                   &priv->tree,
-                                  &priv->keywords,
+                                  &priv->links,
                                   &error)) {
                 if (error != NULL) {
-                        g_warning ("Failed to read '%s': %s",
-                                   index_file_path,
+                        gchar *parse_name;
+
+                        parse_name = g_file_get_parse_name (priv->index_file);
+
+                        g_warning ("Failed to read “%s”: %s",
+                                   parse_name,
                                    error->message);
-                        g_error_free (error);
+
+                        g_free (parse_name);
+                        g_clear_error (&error);
                 }
 
                 /* Deallocate the book, as we are not going to add it in the
@@ -313,12 +314,11 @@ dh_book_new (const gchar *index_file_path)
                 return NULL;
         }
 
-        priv->index_file_path = g_strdup (index_file_path);
-
-        /* Rewrite language, if any, including the prefix we want
-         * to use when seeing it. It is pretty ugly to do it here,
-         * but it's the only way of making sure we standarize how
-         * the language group is shown */
+        /* Rewrite language, if any, including the prefix we want to use when
+         * seeing it, to standarize how the language group is shown.
+         * FIXME: do this in DhLanguage instead (this may require to implement
+         * DhLanguage differently).
+         */
         dh_util_ascii_strtitle (language);
         priv->language = (language != NULL ?
                           g_strdup_printf (_("Language: %s"), language) :
@@ -326,26 +326,38 @@ dh_book_new (const gchar *index_file_path)
         g_free (language);
 
         /* Setup monitor for changes */
-        priv->monitor = g_file_monitor_file (index_file,
-                                             G_FILE_MONITOR_NONE,
-                                             NULL,
-                                             NULL);
-        if (priv->monitor != NULL) {
-                g_signal_connect (priv->monitor,
-                                  "changed",
-                                  G_CALLBACK (book_monitor_event_cb),
-                                  book);
-        } else {
-                g_warning ("Couldn't setup monitoring of changes in book '%s'",
-                           priv->title);
+
+        priv->index_file_monitor = g_file_monitor_file (priv->index_file,
+                                                        G_FILE_MONITOR_NONE,
+                                                        NULL,
+                                                        &error);
+
+        if (error != NULL) {
+                gchar *parse_name;
+
+                parse_name = g_file_get_parse_name (priv->index_file);
+
+                g_warning ("Failed to create file monitor for file “%s”: %s",
+                           parse_name,
+                           error->message);
+
+                g_free (parse_name);
+                g_clear_error (&error);
+        }
+
+        if (priv->index_file_monitor != NULL) {
+                g_signal_connect_object (priv->index_file_monitor,
+                                         "changed",
+                                         G_CALLBACK (index_file_changed_cb),
+                                         book,
+                                         0);
         }
 
-        g_object_unref (index_file);
         return book;
 }
 
 /**
- * dh_book_get_keywords:
+ * dh_book_get_links:
  * @book: a #DhBook.
  *
  * Returns: (element-type DhLink) (transfer none) (nullable): the list of
@@ -353,7 +365,7 @@ dh_book_new (const gchar *index_file_path)
  * disabled.
  */
 GList *
-dh_book_get_keywords (DhBook *book)
+dh_book_get_links (DhBook *book)
 {
         DhBookPrivate *priv;
 
@@ -361,7 +373,7 @@ dh_book_get_keywords (DhBook *book)
 
         priv = dh_book_get_instance_private (book);
 
-        return priv->enabled ? priv->keywords : NULL;
+        return priv->enabled ? priv->links : NULL;
 }
 
 /**
@@ -386,11 +398,11 @@ dh_book_get_completions (DhBook *book)
         if (priv->completions == NULL) {
                 GList *l;
 
-                for (l = priv->keywords; l != NULL; l = l->next) {
+                for (l = priv->links; l != NULL; l = l->next) {
                         DhLink *link = l->data;
                         gchar *str;
 
-                        /* Add additional "page:" and "book:" completions */
+                        /* Add additional "page:" and "book:" completions. */
                         if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
                                 str = g_strdup_printf ("book:%s", dh_link_get_name (link));
                                 priv->completions = g_list_prepend (priv->completions, str);
@@ -416,7 +428,7 @@ dh_book_get_completions (DhBook *book)
  * #DhLink's of type %DH_LINK_TYPE_BOOK or %DH_LINK_TYPE_PAGE. The other
  * #DhLink's are not contained in the tree. To have a list of
  * <emphasis>all</emphasis> #DhLink's part of the book, you need to call
- * dh_book_get_keywords().
+ * dh_book_get_links().
  *
  * Returns: (transfer none) (nullable): the tree of #DhLink's part of the @book,
  * or %NULL if the book is disabled.
@@ -434,13 +446,17 @@ dh_book_get_tree (DhBook *book)
 }
 
 /**
- * dh_book_get_name:
+ * dh_book_get_id:
  * @book: a #DhBook.
  *
- * Returns: the book name.
+ * Gets the book ID. In the Devhelp index file format version 2, it is actually
+ * the “name”, not the ID, but “book ID” is clearer, “book name” can be confused
+ * with the title.
+ *
+ * Returns: the book ID.
  */
 const gchar *
-dh_book_get_name (DhBook *book)
+dh_book_get_id (DhBook *book)
 {
         DhBookPrivate *priv;
 
@@ -448,7 +464,7 @@ dh_book_get_name (DhBook *book)
 
         priv = dh_book_get_instance_private (book);
 
-        return priv->name;
+        return priv->id;
 }
 
 /**
@@ -488,13 +504,13 @@ dh_book_get_language (DhBook *book)
 }
 
 /**
- * dh_book_get_path:
+ * dh_book_get_index_file:
  * @book: a #DhBook.
  *
- * Returns: the path to the index file.
+ * Returns: (transfer none): the index file.
  */
-const gchar *
-dh_book_get_path (DhBook *book)
+GFile *
+dh_book_get_index_file (DhBook *book)
 {
         DhBookPrivate *priv;
 
@@ -502,7 +518,7 @@ dh_book_get_path (DhBook *book)
 
         priv = dh_book_get_instance_private (book);
 
-        return priv->index_file_path;
+        return priv->index_file;
 }
 
 /**
@@ -539,10 +555,13 @@ dh_book_set_enabled (DhBook   *book,
         g_return_if_fail (DH_IS_BOOK (book));
 
         priv = dh_book_get_instance_private (book);
+
+        enabled = enabled != FALSE;
+
         if (priv->enabled != enabled) {
                 priv->enabled = enabled;
                 g_signal_emit (book,
-                               enabled ? signals[BOOK_ENABLED] : signals[BOOK_DISABLED],
+                               enabled ? signals[SIGNAL_BOOK_ENABLED] : signals[SIGNAL_BOOK_DISABLED],
                                0);
         }
 }
@@ -563,6 +582,9 @@ dh_book_cmp_by_path (DhBook *a,
 {
         DhBookPrivate *priv_a;
         DhBookPrivate *priv_b;
+        gchar *path_a;
+        gchar *path_b;
+        gint diff;
 
         if (a == NULL || b == NULL)
                 return -1;
@@ -570,22 +592,29 @@ dh_book_cmp_by_path (DhBook *a,
         priv_a = dh_book_get_instance_private (a);
         priv_b = dh_book_get_instance_private (b);
 
-        return g_strcmp0 (priv_a->index_file_path, priv_b->index_file_path);
+        path_a = g_file_get_path (priv_a->index_file);
+        path_b = g_file_get_path (priv_b->index_file);
+
+        diff = g_strcmp0 (path_a, path_b);
+
+        g_free (path_a);
+        g_free (path_b);
+        return diff;
 }
 
 /**
- * dh_book_cmp_by_name:
+ * dh_book_cmp_by_id:
  * @a: a #DhBook.
  * @b: a #DhBook.
  *
- * Compares the #DhBook's by their name.
+ * Compares the #DhBook's by their IDs, with g_ascii_strcasecmp().
  *
  * Returns: an integer less than, equal to, or greater than zero, if @a is <, ==
  * or > than @b.
  */
 gint
-dh_book_cmp_by_name (DhBook *a,
-                     DhBook *b)
+dh_book_cmp_by_id (DhBook *a,
+                   DhBook *b)
 {
         DhBookPrivate *priv_a;
         DhBookPrivate *priv_b;
@@ -596,10 +625,10 @@ dh_book_cmp_by_name (DhBook *a,
         priv_a = dh_book_get_instance_private (a);
         priv_b = dh_book_get_instance_private (b);
 
-        if (priv_a->name == NULL || priv_b->name == NULL)
+        if (priv_a->id == NULL || priv_b->id == NULL)
                 return -1;
 
-        return g_ascii_strcasecmp (priv_a->name, priv_b->name);
+        return g_ascii_strcasecmp (priv_a->id, priv_b->id);
 }
 
 /**
