@@ -15,14 +15,14 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _DH_BOOK_H_
-#define _DH_BOOK_H_
+#ifndef DH_BOOK_H
+#define DH_BOOK_H
 
-#include <glib-object.h>
+#include <gio/gio.h>
 
 G_BEGIN_DECLS
 
@@ -45,28 +45,37 @@ struct _DhBookClass {
 };
 
 GType        dh_book_get_type        (void) G_GNUC_CONST;
-DhBook      *dh_book_new             (const gchar  *book_path);
-GList       *dh_book_get_keywords    (DhBook *book);
-GList       *dh_book_get_completions (DhBook *book);
-GNode       *dh_book_get_tree        (DhBook *book);
-const gchar *dh_book_get_name        (DhBook *book);
+
+DhBook *     dh_book_new             (GFile *index_file);
+
+GList *      dh_book_get_links       (DhBook *book);
+
+GList *      dh_book_get_completions (DhBook *book);
+
+GNode *      dh_book_get_tree        (DhBook *book);
+
+const gchar *dh_book_get_id          (DhBook *book);
+
 const gchar *dh_book_get_title       (DhBook *book);
+
 const gchar *dh_book_get_language    (DhBook *book);
-const gchar *dh_book_get_path        (DhBook *book);
+
+GFile *      dh_book_get_index_file  (DhBook *book);
+
 gboolean     dh_book_get_enabled     (DhBook *book);
-void         dh_book_set_enabled     (DhBook *book,
-                                      gboolean enabled);
+
+void         dh_book_set_enabled     (DhBook   *book,
+                                      gboolean  enabled);
+
 gint         dh_book_cmp_by_path     (DhBook *a,
                                       DhBook *b);
-gint         dh_book_cmp_by_path_str (DhBook *a,
-                                      const gchar  *b_path);
-gint         dh_book_cmp_by_name     (DhBook *a,
+
+gint         dh_book_cmp_by_id       (DhBook *a,
                                       DhBook *b);
-gint         dh_book_cmp_by_name_str (DhBook *a,
-                                      const gchar  *b_name);
+
 gint         dh_book_cmp_by_title    (DhBook *a,
                                       DhBook *b);
 
 G_END_DECLS
 
-#endif /* _DH_BOOK_H_ */
+#endif /* DH_BOOK_H */
