@@ -16,16 +16,14 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "dh-keyword-model.h"
-
 #include <gtk/gtk.h>
 #include <string.h>
-
 #include "dh-book.h"
 #include "dh-util.h"
 
@@ -45,9 +43,12 @@ typedef struct {
         gchar *current_book_id;
 
         /* List of DhLink* */
+        /* FIXME ref the DhLinks, in case a DhBook is destroyed while the
+         * DhLinks are still stored here.
+         */
         GQueue keywords;
 
-        gint   stamp;
+        gint stamp;
 } DhKeywordModelPrivate;
 
 /* Store a keyword as well as glob
@@ -68,7 +69,6 @@ typedef struct {
         const gchar *book_id;
         const gchar *skip_book_id;
         const gchar *page_id;
-        gchar *page_filename_prefix;
         const gchar *language;
         guint case_sensitive : 1;
         guint prefix : 1;
@@ -76,7 +76,7 @@ typedef struct {
 
 #define MAX_HITS 100
 
-static void dh_keyword_model_tree_model_init (GtkTreeModelIface   *iface);
+static void dh_keyword_model_tree_model_init (GtkTreeModelIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (DhKeywordModel, dh_keyword_model, G_TYPE_OBJECT,
                          G_ADD_PRIVATE (DhKeywordModel)
@@ -97,7 +97,7 @@ dh_keyword_model_finalize (GObject *object)
 static void
 dh_keyword_model_class_init (DhKeywordModelClass *klass)
 {
-        GObjectClass *object_class = G_OBJECT_CLASS (klass);;
+        GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
         object_class->finalize = dh_keyword_model_finalize;
 }
@@ -113,6 +113,7 @@ dh_keyword_model_init (DhKeywordModel *model)
 static GtkTreeModelFlags
 keyword_model_get_flags (GtkTreeModel *tree_model)
 {
+        /* FIXME: check if GTK_TREE_MODEL_ITERS_PERSIST is correct. */
         return GTK_TREE_MODEL_ITERS_PERSIST | GTK_TREE_MODEL_LIST_ONLY;
 }
 
@@ -131,6 +132,10 @@ keyword_model_get_column_type (GtkTreeModel *tree_model,
                 return G_TYPE_STRING;
 
         case DH_KEYWORD_MODEL_COL_LINK:
+                /* FIXME: use DH_TYPE_LINK boxed type, to take advantage of ref
+                 * counting, to have safer code in case a DhLink is freed when
+                 * still stored in the GtkTreeModel.
+                 */
                 return G_TYPE_POINTER;
 
         case DH_KEYWORD_MODEL_COL_CURRENT_BOOK_FLAG:
@@ -443,7 +448,7 @@ keyword_model_search_book (DhBook          *book,
 
         ret = g_queue_new ();
 
-        for (l = dh_book_get_keywords (book);
+        for (l = dh_book_get_links (book);
              l != NULL && ret->length < max_hits;
              l = g_list_next (l)) {
                 DhLink   *link;
@@ -454,20 +459,12 @@ keyword_model_search_book (DhBook          *book,
 
                 /* Filter by page? */
                 if (settings->page_id != NULL) {
-                        gchar *file_name;
-
-                        file_name = (settings->case_sensitive ?
-                                     g_strdup (dh_link_get_file_name (link)) :
-                                     g_ascii_strdown (dh_link_get_file_name (link), -1));
-
-                        /* First, filter out all keywords not belonging
-                         * to this given page. */
-                        if (!g_str_has_prefix (file_name, settings->page_filename_prefix)) {
+                        if (!dh_link_belongs_to_page (link,
+                                                      settings->page_id,
+                                                      settings->case_sensitive)) {
                                 /* No need of this keyword. */
-                                g_free (file_name);
                                 continue;
                         }
-                        g_free (file_name);
 
                         /* This means we got no keywords to look for. */
                         if (settings->keywords == NULL) {
@@ -591,7 +588,7 @@ keyword_model_search_books (DhKeywordModel  *model,
 
                 /* Filtering by book? */
                 if (settings->book_id != NULL) {
-                        if (g_strcmp0 (settings->book_id, dh_book_get_name (book)) != 0) {
+                        if (g_strcmp0 (settings->book_id, dh_book_get_id (book)) != 0) {
                                 continue;
                         }
 
@@ -615,7 +612,7 @@ keyword_model_search_books (DhKeywordModel  *model,
 
                 /* Skipping a given book? */
                 if (settings->skip_book_id != NULL &&
-                    g_strcmp0 (settings->skip_book_id, dh_book_get_name (book)) == 0) {
+                    g_strcmp0 (settings->skip_book_id, dh_book_get_id (book)) == 0) {
                         continue;
                 }
 
@@ -661,14 +658,11 @@ keyword_model_search (DhKeywordModel  *model,
         settings.book_id = book_id;
         settings.skip_book_id = NULL;
         settings.page_id = page_id;
-        settings.page_filename_prefix = NULL;
         settings.language = language;
         settings.case_sensitive = case_sensitive;
         settings.prefix = TRUE;
 
         if (page_id != NULL) {
-                settings.page_filename_prefix = g_strdup_printf ("%s.", page_id);
-
                 /* If filtering per page, increase the maximum number of
                  * hits. This is due to the fact that a page may have
                  * more than MAX_HITS keywords, and the page link may be
@@ -742,8 +736,6 @@ keyword_model_search (DhKeywordModel  *model,
 
 out:
         dh_globbed_keywords_free (settings.keyword_globs);
-        g_free (settings.page_filename_prefix);
-
         return out;
 }
 
