@@ -1,7 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2001-2002 Mikael Hallendal <micke@imendio.com>
- * Copyright (C) 2008      Imendio AB
+ * Copyright (C) 2008 Imendio AB
+ * Copyright (C) 2017 Sébastien Wilmet <swilmet@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,102 +37,161 @@
  * There is exactly one #DhLink of type %DH_LINK_TYPE_BOOK per #DhBook object.
  */
 
+/* Fields used only by DhLink's of type DH_LINK_TYPE_BOOK. */
+typedef struct {
+        gchar *base_path;
+        gchar *book_id;
+} BookData;
+
 struct _DhLink {
-        /* FIXME: Those two could exist only for book to save some
-         * memory.
+        /* To avoid some memory padding inside the struct, to use less memory,
+         * the fields are placed in this order:
+         * 1. All the pointers.
+         * 2. Other types.
+         * 3. Bit fields.
+         *
+         * Also, a union is used to use less memory. This struct is allocated a
+         * lot, so it is worth optimizing it.
          */
-        gchar       *book_id;
-        gchar       *base_path;
 
-        DhLink      *book;
+        union {
+                /* @book.data is set only for links of @type DH_LINK_TYPE_BOOK. */
+                BookData *data;
 
-        gchar       *name;
-        gchar       *name_collation_key;
-        gchar       *relative_url;
+                /* @book.link is set only for links of @type != DH_LINK_TYPE_BOOK. */
+                DhLink *link;
+        } book;
 
-        guint        ref_count;
+        gchar *name;
+        gchar *name_collation_key;
 
-        DhLinkType   type : 8;
-        DhLinkFlags  flags : 8;
+        gchar *relative_url;
+
+        guint ref_count;
+
+        DhLinkType type : 8;
+        DhLinkFlags flags : 8;
 };
 
+/* If the relative_url is empty. */
+#define DEFAULT_PAGE "index.html"
+
 G_DEFINE_BOXED_TYPE (DhLink, dh_link,
                      dh_link_ref, dh_link_unref)
 
+static BookData *
+book_data_new (const gchar *base_path,
+               const gchar *book_id)
+{
+        BookData *data;
+
+        data = g_slice_new (BookData);
+        data->base_path = g_strdup (base_path);
+        data->book_id = g_strdup (book_id);
+
+        return data;
+}
+
+static void
+book_data_free (BookData *data)
+{
+        if (data == NULL)
+                return;
+
+        g_free (data->base_path);
+        g_free (data->book_id);
+        g_slice_free (BookData, data);
+}
+
 static void
 link_free (DhLink *link)
 {
-        g_free (link->book_id);
-        g_free (link->base_path);
+        if (link->type == DH_LINK_TYPE_BOOK)
+                book_data_free (link->book.data);
+        else
+                dh_link_unref (link->book.link);
+
         g_free (link->name);
         g_free (link->name_collation_key);
         g_free (link->relative_url);
 
-        if (link->book != NULL)
-                dh_link_unref (link->book);
-
         g_slice_free (DhLink, link);
 }
 
+static DhLink *
+dh_link_new_common (DhLinkType   type,
+                    const gchar *name,
+                    const gchar *relative_url)
+{
+        DhLink *link;
+
+        link = g_slice_new0 (DhLink);
+        link->ref_count = 1;
+        link->type = type;
+        link->name = g_strdup (name);
+        link->relative_url = g_strdup (relative_url);
+
+        return link;
+}
+
+/**
+ * dh_link_new_book:
+ * @base_path: the base path for the book.
+ * @book_id: the book ID.
+ * @book_title: the name of the link.
+ * @relative_url: the URL relative to the book @base_path. Can contain an
+ * anchor. Usually the index.html page.
+ *
+ * Returns: a new #DhLink of type %DH_LINK_TYPE_BOOK.
+ * Since: 3.28
+ */
+DhLink *
+dh_link_new_book (const gchar *base_path,
+                  const gchar *book_id,
+                  const gchar *book_title,
+                  const gchar *relative_url)
+{
+        DhLink *link;
+
+        g_return_val_if_fail (base_path != NULL, NULL);
+        g_return_val_if_fail (book_id != NULL, NULL);
+        g_return_val_if_fail (book_title != NULL, NULL);
+        g_return_val_if_fail (relative_url != NULL, NULL);
+
+        link = dh_link_new_common (DH_LINK_TYPE_BOOK, book_title, relative_url);
+
+        link->book.data = book_data_new (base_path, book_id);
+
+        return link;
+}
+
 /**
  * dh_link_new:
- * @type: the #DhLinkType.
- * @base_path: (nullable): the base path for the book, or %NULL.
- * @book_id: (nullable): the book ID, or %NULL.
+ * @type: the #DhLinkType. Must be different than %DH_LINK_TYPE_BOOK.
+ * @book_link: the #DhLink of type %DH_LINK_TYPE_BOOK for the book that the link
+ *   is contained in.
  * @name: the name of the link.
- * @book: (nullable): the book that the link is contained in, or %NULL.
- * @relative_url: the URL relative to the book @base_path. Can contain an
- * anchor.
- *
- * Creates a new #DhLink.
- *
- * @base_path and @book_id must be provided only for a link of type
- * %DH_LINK_TYPE_BOOK.
- *
- * If @type is not a #DH_LINK_TYPE_BOOK, then the @book link must be provided.
- *
- * @name and @relative_url must always be provided.
+ * @relative_url: the URL relative to the book base path. Can contain an anchor.
  *
  * Returns: a new #DhLink.
  */
 DhLink *
 dh_link_new (DhLinkType   type,
-             const gchar *base_path,
-             const gchar *book_id,
-             DhLink      *book,
+             DhLink      *book_link,
              const gchar *name,
              const gchar *relative_url)
 {
         DhLink *link;
 
-        if (type == DH_LINK_TYPE_BOOK) {
-                g_return_val_if_fail (base_path != NULL, NULL);
-                g_return_val_if_fail (book_id != NULL, NULL);
-                g_return_val_if_fail (book == NULL, NULL);
-        } else {
-                g_return_val_if_fail (base_path == NULL, NULL);
-                g_return_val_if_fail (book_id == NULL, NULL);
-                g_return_val_if_fail (book != NULL, NULL);
-        }
-
+        g_return_val_if_fail (type != DH_LINK_TYPE_BOOK, NULL);
+        g_return_val_if_fail (book_link != NULL, NULL);
+        g_return_val_if_fail (book_link->type == DH_LINK_TYPE_BOOK, NULL);
         g_return_val_if_fail (name != NULL, NULL);
         g_return_val_if_fail (relative_url != NULL, NULL);
 
-        link = g_slice_new0 (DhLink);
+        link = dh_link_new_common (type, name, relative_url);
 
-        link->ref_count = 1;
-        link->type = type;
-
-        if (type == DH_LINK_TYPE_BOOK) {
-                link->base_path = g_strdup (base_path);
-                link->book_id = g_strdup (book_id);
-        }
-
-        if (book != NULL)
-                link->book = dh_link_ref (book);
-
-        link->name = g_strdup (name);
-        link->relative_url = g_strdup (relative_url);
+        link->book.link = dh_link_ref (book_link);
 
         return link;
 }
@@ -142,6 +202,8 @@ dh_link_new (DhLinkType   type,
  *
  * Increases the reference count of @link.
  *
+ * Not thread-safe.
+ *
  * Returns: (transfer full): the @link.
  */
 DhLink *
@@ -149,7 +211,7 @@ dh_link_ref (DhLink *link)
 {
         g_return_val_if_fail (link != NULL, NULL);
 
-        g_atomic_int_inc (&link->ref_count);
+        link->ref_count++;
 
         return link;
 }
@@ -159,16 +221,18 @@ dh_link_ref (DhLink *link)
  * @link: a #DhLink.
  *
  * Decreases the reference count of @link.
+ *
+ * Not thread-safe.
  */
 void
 dh_link_unref (DhLink *link)
 {
         g_return_if_fail (link != NULL);
 
-        if (g_atomic_int_dec_and_test (&link->ref_count))
-        {
+        if (link->ref_count == 1)
                 link_free (link);
-        }
+        else
+                link->ref_count--;
 }
 
 /**
@@ -219,7 +283,8 @@ dh_link_set_flags (DhLink      *link,
  * dh_link_get_name:
  * @link: a #DhLink.
  *
- * Returns: the name of the @link.
+ * Returns: the name of the @link. For a link of type %DH_LINK_TYPE_BOOK,
+ * returns the book title.
  */
 const gchar *
 dh_link_get_name (DhLink *link)
@@ -230,30 +295,102 @@ dh_link_get_name (DhLink *link)
 }
 
 /**
- * dh_link_get_file_name:
+ * dh_link_match_relative_url:
  * @link: a #DhLink.
+ * @relative_url: an URL relative to the book base path. Can contain an anchor.
  *
- * Returns: the name of the file that the @link is contained in.
+ * Returns: whether the relative URL of @link matches with @relative_url. There
+ * is a special case for the index.html page, it can also match the empty
+ * string.
+ * Since: 3.28
  */
-const gchar *
-dh_link_get_file_name (DhLink *link)
+gboolean
+dh_link_match_relative_url (DhLink      *link,
+                            const gchar *relative_url)
 {
-        g_return_val_if_fail (link != NULL, NULL);
+        g_return_val_if_fail (link != NULL, FALSE);
+        g_return_val_if_fail (link->relative_url != NULL, FALSE);
+        g_return_val_if_fail (relative_url != NULL, FALSE);
 
-        /* Return filename if the link is itself a page or if the link is within
-         * a page (i.e. every link type except a book).
+        if (g_str_equal (link->relative_url, relative_url))
+                return TRUE;
+
+        /* Special case for index.html, can also match the empty string.
+         * Example of full URLs:
+         * file:///usr/share/gtk-doc/html/glib/
+         * file:///usr/share/gtk-doc/html/glib/index.html
+         *
+         * This supports only the root index.html page of a DhBook, this doesn't
+         * support index.html inside a sub-directory, if the relative_url
+         * contains a sub-directory. But apparently GTK-Doc doesn't create
+         * sub-directories, all the *.html pages are in the same directory.
          */
-        if (link->type != DH_LINK_TYPE_BOOK)
-                return link->relative_url;
+        if (relative_url[0] == '\0')
+                return g_str_equal (link->relative_url, DEFAULT_PAGE);
+        else if (link->relative_url[0] == '\0')
+                return g_str_equal (relative_url, DEFAULT_PAGE);
 
-        return "";
+        return FALSE;
+}
+
+/**
+ * dh_link_belongs_to_page:
+ * @link: a #DhLink.
+ * @page_id: a page ID, i.e. the filename without its extension.
+ * @case_sensitive: whether @page_id is case sensitive.
+ *
+ * This function permits to know if @link belongs to a certain page.
+ *
+ * @page_id is usually the HTML filename without the `.html` extension. More
+ * generally, @page_id must be a relative URL (relative to the book base path),
+ * without the anchor nor the file extension.
+ *
+ * For example if @link has the relative URL `"DhLink.html#dh-link-ref"`, then
+ * this function will return %TRUE if the @page_id is `"DhLink"`.
+ *
+ * Returns: whether @link belongs to @page_id.
+ * Since: 3.28
+ */
+gboolean
+dh_link_belongs_to_page (DhLink      *link,
+                         const gchar *page_id,
+                         gboolean     case_sensitive)
+{
+        const gchar *relative_url;
+        gsize page_id_len;
+        gboolean has_prefix;
+
+        g_return_val_if_fail (link != NULL, FALSE);
+        g_return_val_if_fail (link->relative_url != NULL, FALSE);
+        g_return_val_if_fail (page_id != NULL, FALSE);
+
+        relative_url = link->relative_url;
+        if (relative_url[0] == '\0')
+                relative_url = DEFAULT_PAGE;
+
+        page_id_len = strlen (page_id);
+
+        if (case_sensitive)
+                has_prefix = strncmp (relative_url, page_id, page_id_len) == 0;
+        else
+                has_prefix = g_ascii_strncasecmp (relative_url, page_id, page_id_len) == 0;
+
+        /* Check that a file extension follows. */
+        if (has_prefix)
+                return relative_url[page_id_len] == '.';
+
+        return FALSE;
 }
 
 /**
  * dh_link_get_uri:
  * @link: a #DhLink.
  *
- * Returns: the @link URI. Free with g_free() when no longer needed.
+ * Gets the @link URI, by concateneting the book base path with the @link
+ * relative URL.
+ *
+ * Returns: (nullable): the @link URI, or %NULL if getting the URI failed. Free
+ * with g_free() when no longer needed.
  */
 gchar *
 dh_link_get_uri (DhLink *link)
@@ -262,13 +399,14 @@ dh_link_get_uri (DhLink *link)
         gchar *filename;
         gchar *uri;
         gchar *anchor;
+        GError *error = NULL;
 
         g_return_val_if_fail (link != NULL, NULL);
 
         if (link->type == DH_LINK_TYPE_BOOK)
-                base_path = link->base_path;
+                base_path = link->book.data->base_path;
         else
-                base_path = link->book->base_path;
+                base_path = link->book.link->book.data->base_path;
 
         filename = g_build_filename (base_path, link->relative_url, NULL);
 
@@ -278,9 +416,13 @@ dh_link_get_uri (DhLink *link)
                 anchor++;
         }
 
-        uri = g_filename_to_uri (filename, NULL, NULL);
+        uri = g_filename_to_uri (filename, NULL, &error);
+        if (error != NULL) {
+                g_warning ("Failed to get DhLink URI: %s", error->message);
+                g_clear_error (&error);
+        }
 
-        if (anchor != NULL) {
+        if (uri != NULL && anchor != NULL) {
                 gchar *uri_with_anchor;
 
                 uri_with_anchor = g_strconcat (uri, "#", anchor, NULL);
@@ -293,18 +435,21 @@ dh_link_get_uri (DhLink *link)
 }
 
 /**
- * dh_link_get_book_name:
+ * dh_link_get_book_title:
  * @link: a #DhLink.
  *
- * Returns: the name of the book that the @link is contained in.
+ * Returns: the title of the book that the @link is contained in.
  */
 const gchar *
-dh_link_get_book_name (DhLink *link)
+dh_link_get_book_title (DhLink *link)
 {
         g_return_val_if_fail (link != NULL, NULL);
 
-        if (link->book != NULL)
-                return link->book->name;
+        if (link->type == DH_LINK_TYPE_BOOK)
+                return link->name;
+
+        if (link->book.link != NULL)
+                return link->book.link->name;
 
         return "";
 }
@@ -313,7 +458,7 @@ dh_link_get_book_name (DhLink *link)
  * dh_link_get_book_id:
  * @link: a #DhLink.
  *
- * Returns: the book ID.
+ * Returns: the ID of the book that the @link is contained in.
  */
 const gchar *
 dh_link_get_book_id (DhLink *link)
@@ -321,14 +466,38 @@ dh_link_get_book_id (DhLink *link)
         g_return_val_if_fail (link != NULL, NULL);
 
         if (link->type == DH_LINK_TYPE_BOOK)
-                return link->book_id;
+                return link->book.data->book_id;
 
-        if (link->book != NULL)
-                return link->book->book_id;
+        if (link->book.link != NULL)
+                return link->book.link->book.data->book_id;
 
         return "";
 }
 
+static gint
+dh_link_type_compare (DhLinkType a,
+                      DhLinkType b)
+{
+        if (a == b)
+                return 0;
+
+        /* Same order as in a tree: first the top-level book node, then pages,
+         * then keywords (keywords are contained in a page).
+         */
+
+        if (a == DH_LINK_TYPE_BOOK)
+                return -1;
+        if (b == DH_LINK_TYPE_BOOK)
+                return 1;
+
+        if (a == DH_LINK_TYPE_PAGE)
+                return -1;
+        if (b == DH_LINK_TYPE_PAGE)
+                return 1;
+
+        return 0;
+}
+
 /**
  * dh_link_compare:
  * @a: (type DhLink): a #DhLink.
@@ -368,23 +537,10 @@ dh_link_compare (gconstpointer a,
         diff = strcmp (la->name_collation_key,
                        lb->name_collation_key);
 
-        /* For the same names, sort page links before other links. The page is
-         * more important than a symbol (typically contained in that page).
-         */
-        if (diff == 0) {
-                if (la->type == lb->type)
-                        return 0;
+        if (diff != 0)
+                return diff;
 
-                if (la->type == DH_LINK_TYPE_PAGE)
-                        return -1;
-
-                if (lb->type == DH_LINK_TYPE_PAGE)
-                        return 1;
-
-                return 0;
-        }
-
-        return diff;
+        return dh_link_type_compare (la->type, lb->type);
 }
 
 /**
