@@ -30,11 +30,14 @@
  * one day there is a strong desire to create a new version):
  * - Replace <functions> element by <keywords>.
  * - Maybe have an up-to-date URI for the NAMESPACE.
- * - Rename some attributes of <book>:
- *      - name -> id
- *      - title -> name
- *   Because with version 2 it's confusing that dh_link_get_name() returns the
- *   title, and dh_link_get_book_id() returns the book name.
+ * - Rename <book> attribute 'name' to 'id', because "book name" can be confused
+ *   with the book title or other 'name' attributes (for <sub> and <keyword>,
+ *   the 'name' attribute has a different meaning). With "book ID" there is no
+ *   ambiguity.
+ * - Maybe rename <book> attribute 'title' to 'name', to be consistent with
+ *   the <sub> and <keyword> elements. dh_link_get_name() would also have a
+ *   clearer meaning for book top-level links. But "book title" has the
+ *   advantage that there is no ambiguity.
  */
 
 /* It's the xmlns attribute. It is currently (well, in 2015 at least) used on
@@ -61,7 +64,7 @@ typedef struct {
         GFile *index_file;
 
         gchar *book_title;
-        gchar *book_name;
+        gchar *book_id;
         gchar *book_language;
 
         /* List of all DhLink* */
@@ -91,7 +94,7 @@ dh_parser_free (DhParser *parser)
         g_clear_object (&parser->index_file);
 
         g_free (parser->book_title);
-        g_free (parser->book_name);
+        g_free (parser->book_id);
         g_free (parser->book_language);
 
         g_list_free_full (parser->all_links, (GDestroyNotify)dh_link_unref);
@@ -187,8 +190,8 @@ parser_start_node_book (DhParser             *parser,
         parser->book_title = g_strdup (title);
         replace_newlines_by_spaces (parser->book_title);
 
-        g_free (parser->book_name);
-        parser->book_name = g_strdup (name);
+        g_free (parser->book_id);
+        parser->book_id = g_strdup (name);
 
         g_free (parser->book_language);
         parser->book_language = g_strdup (language);
@@ -201,12 +204,10 @@ parser_start_node_book (DhParser             *parser,
                 g_object_unref (directory);
         }
 
-        link = dh_link_new (DH_LINK_TYPE_BOOK,
-                            base,
-                            parser->book_name,
-                            NULL,
-                            parser->book_title,
-                            uri);
+        link = dh_link_new_book (base,
+                                 parser->book_id,
+                                 parser->book_title,
+                                 uri);
         g_free (base);
         parser->all_links = g_list_prepend (parser->all_links, link);
 
@@ -264,8 +265,6 @@ parser_start_node_chapter (DhParser             *parser,
         g_assert (parser->book_node != NULL);
 
         link = dh_link_new (DH_LINK_TYPE_PAGE,
-                            NULL,
-                            NULL,
                             parser->book_node->data,
                             name,
                             uri);
@@ -428,8 +427,6 @@ parser_start_node_keyword (DhParser             *parser,
         g_assert (parser->book_node != NULL);
 
         link = dh_link_new (link_type,
-                            NULL,
-                            NULL,
                             parser->book_node->data,
                             name,
                             uri);
@@ -525,7 +522,7 @@ parser_end_node_cb (GMarkupParseContext  *context,
 gboolean
 dh_parser_read_file (GFile   *index_file,
                      gchar  **book_title,
-                     gchar  **book_name,
+                     gchar  **book_id,
                      gchar  **book_language,
                      GNode  **book_tree,
                      GList  **all_links,
@@ -540,7 +537,7 @@ dh_parser_read_file (GFile   *index_file,
 
         g_return_val_if_fail (G_IS_FILE (index_file), FALSE);
         g_return_val_if_fail (book_title != NULL && *book_title == NULL, FALSE);
-        g_return_val_if_fail (book_name != NULL && *book_name == NULL, FALSE);
+        g_return_val_if_fail (book_id != NULL && *book_id == NULL, FALSE);
         g_return_val_if_fail (book_language != NULL && *book_language == NULL, FALSE);
         g_return_val_if_fail (book_tree != NULL && *book_tree == NULL, FALSE);
         g_return_val_if_fail (all_links != NULL && *all_links == NULL, FALSE);
@@ -627,8 +624,8 @@ dh_parser_read_file (GFile   *index_file,
         *book_title = parser->book_title;
         parser->book_title = NULL;
 
-        *book_name = parser->book_name;
-        parser->book_name = NULL;
+        *book_id = parser->book_id;
+        parser->book_id = NULL;
 
         *book_language = parser->book_language;
         parser->book_language = NULL;
